module dsim1 
#(parameter DATA_WIDTH=16, parameter ADDR_WIDTH=8)
(
	input clk, reset_n,
	output [(ADDR_WIDTH-1):0] addr,
	output we,
	output [2:0] ra, rb, rd,
	output  [(DATA_WIDTH-1):0] q, qa, qb, qd,
	output reg [7:0] pc,
	output reg [(DATA_WIDTH-1):0] inst,
	output [4:0] opcode,
	output ma, rwe
);

single_port_ram dev1(
	.data(qb),
	.addr(addr[6:0]),
	.we(we),
	.clk(clk),
	.q(q)
);

regfile dev2(
	.clk(clk),
	.ra(ra), .rb(rb), .rd(rd),
	.qa(qa), .qb(qb), .qd(qd),
	.we(rwe)
);

//reg [7:0] pc;
assign rwe = state==WB;


//wire [2:0] opcode = q[15:13];
//wire [4:0] opcode;

wire [15:0] qi = (state==ID? q: inst);
assign opcode = qi[15:11];
assign rd = (op_sw? 3'h0: qi[10:8]);
assign ra = (ma? 3'd0: qi[7:5]);
assign rb = (op_lw? qi[10:8]: qi[4:2]);
wire [7:0] immed = qi[7:0];
	
reg [2:0] state;
localparam 
	IF = 3'h0,
	ID = 3'h1,
	EX = 3'h2,
	MEM = 3'h3,
	WB = 3'h4,
	HALT = 3'h7;
	
`include "opcodes.txt"

wire op_sw = opcode==SW;
wire op_lw = opcode==LW;

assign we = (state==MEM) && op_sw;
assign ma = (op_sw || op_lw); // memory_access

wire [7:0] nextpc = pc + 1'b1;

assign addr = (state==MEM? immed: nextpc);


// writeback register multiplexer
assign qd = (op_lw? q: 16'h0000);


always @(posedge clk, negedge reset_n)
begin
  if (!reset_n)
  begin
   pc <= 8'hFF;
   state <= IF;
   end
  else 
	case (state)
	IF:
	begin
		state <= ID;
		pc <= addr;
    end
    ID:
    begin
	 // save results of IF phase
	 inst <= q;
	 state <= EX;
	end
	EX:
	begin
	  if (inst == 16'h0000) state <= HALT;
      else if (ma)
		state <= MEM;
      else 
		state <= WB;
	end
   MEM:
    begin
	   state <= WB;
	end
	WB:
	begin
	   state <= IF;
	end
	HALT:
	begin
		state <= HALT;
	end
	default:
		state <= HALT;
  endcase
end


endmodule